/*
  Module EEPROM / SD
  part of Arduino Mega Server project
*/

#define MAX_ID_DIGS 15
byte buffId[MAX_ID_DIGS];

File eeFile;
char eeName[] = "EEPROM.DAT";
#define MAX_EEPROM 2048

#define SD_ADR_ADDRESS  100
#define SD_ID_ADR       102
#define SD_ID_LEN       15
#define SD_ID_CHECK_ADR 117
#define SD_ID_CHECK_VAL 231
#define SD_NAME_ADR     118
#define SD_NAME_LEN     15

// init

void sdConfigInit() {
  initStart(F("EEPROM /SD"));
  Serial.print(F(" EEPROM.DAT: "));
  if (existConfig()) {
    Serial.println(F("exist"));
  } else {
      clearConfig();
      Serial.println(F("created"));
    }
  if (!sdCheckName()) {
    sdSetName(SELF_NAME);
  }
  if (!sdGetAddress()) {
    sdSetAddress(random(1, 255));
  }
  sdConfigCheckId();
  printSdConfigInfo();
  initDone();
}

// Config file

bool openReadConfig() {
  eeFile = SD.open(String (eeName));
  if (eeFile) {
    return true;
  } else {
      return false;
    }
}

bool openWriteConfig() {
  eeFile = SD.open(String (eeName), FILE_WRITE);
  if (eeFile) {
    return true;
  } else {
      return false;
    }
}

void closeConfig() {
  eeFile.close(); 
}

bool existConfig() {
  if (SD.exists(eeName)) {
    return true;
  } else {
      return false;
    }
}

void deleteConfig() {
  if (SD.exists(eeName)) {
    SD.remove(eeName);
  }
}

void clearConfig() {
  byte zero = 0;
  eeFile = SD.open(eeName, FILE_WRITE);
  for (int i = 0; i < MAX_EEPROM; i++) {
    eeFile.write(zero);
  }
  eeFile.close();
}

void printConfig() {
  int newLine = 0;
  eeFile = SD.open(eeName);
  if (eeFile) {
    for (int i = 0; i < MAX_EEPROM; i++) {
      byte b = eeFile.read();
      if (b != 0) {
        Serial.write(b);
      } else {
          Serial.print(F("."));
        }
      newLine++;
      if (newLine > 63) {
        newLine = 0;
        Serial.println();
      }
    }
    eeFile.close();
  } else {
    Serial.println("Config file not exist");
    }
}

// Address

void sdSetAddress(byte addr) {
  if (existConfig()) {
    if (openWriteConfig()) {
      eeFile.seek(SD_ADR_ADDRESS);
      eeFile.write(addr);
      closeConfig();  
    }
  }
  
}

byte sdGetAddress() {
  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_ADR_ADDRESS);
      byte b = eeFile.read(); 
      closeConfig();
      return b;
    }
  }
  return 0;
}

// Name

bool sdCheckName() {
  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_NAME_ADR);
      for (int i = SD_NAME_ADR; i < SD_NAME_ADR + SD_NAME_LEN; i++) {
        if (eeFile.read() != 0) {
          closeConfig();
          return true;
          }
      }
    }
  }
  return false;
}

void sdSetName(String s) {
  if (existConfig()) {
    if (openWriteConfig()) {
      eeFile.seek(SD_NAME_ADR);
      for (int i = SD_NAME_ADR; i < SD_NAME_ADR + SD_NAME_LEN; i++) {
        if (i - SD_NAME_ADR > s.length()) {break;}
        eeFile.write(s[i - SD_NAME_ADR]);
      }
      closeConfig();
    }
  }
}


void sdPrintName(String prefix, String suffix) {
  Serial.print(prefix);
  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_NAME_ADR);
      for (int i = SD_NAME_ADR; i < SD_NAME_ADR + SD_NAME_LEN; i++) {
        Serial.write(eeFile.read());
      }
      closeConfig(); 
    }
  }
  Serial.print(suffix);
}

String sdStringName() {
  String s = "";

  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_NAME_ADR);
      for (int i = SD_NAME_ADR; i < SD_NAME_ADR + SD_NAME_LEN; i++) {
        byte b = eeFile.read();
        if (b == 0) {
          break;
        }
        s += char(b);
      }
      closeConfig(); 
    }
  }
  return s;
}

// ID

bool sdCorrectId() {
  byte current, back;
  byte match    = 0;
  byte increase = 0;
  byte decrease = 0;

  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_ID_ADR);
      for (int i = SD_ID_ADR; i < SD_ID_ADR + SD_ID_LEN; i++) {
        current = eeFile.read();
        if (!validChar(current)) {closeConfig(); return false;}
        if (current == back)     {match++;    if (match    > 5) {closeConfig(); return false;}}
        if (current == back + 1) {increase++; if (increase > 5) {closeConfig(); return false;}}
        if (current == back - 1) {decrease++; if (decrease > 5) {closeConfig(); return false;}}
        back = current;
      }
      eeFile.seek(SD_ID_CHECK_ADR);
      if (eeFile.read() != SD_ID_CHECK_VAL) {
        closeConfig();
        return false;
      }
    }
  }
  return true;
}

void sdSetId() {
  if (existConfig()) {
    if (openWriteConfig()) {
      eeFile.seek(SD_ID_ADR);
      for (int i = SD_ID_ADR; i < SD_ID_ADR + SD_ID_LEN; i++) {
        eeFile.write(buffId[i - SD_ID_ADR]);
      }
      eeFile.seek(SD_ID_CHECK_ADR);
      eeFile.write(SD_ID_CHECK_VAL);
      closeConfig();  
    }
  }
}

void sdSetBuffId() {
  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_ID_ADR);
      for (int i = SD_ID_ADR; i < SD_ID_ADR + SD_ID_LEN; i++) {
        buffId[i - SD_ID_ADR] = eeFile.read();
      }
      closeConfig();  
    }
  }
}

void sdPrintId(String prefix, String suffix) {
  Serial.print(prefix);
  if (existConfig()) {
    if (openReadConfig()) {
      eeFile.seek(SD_ID_ADR);
      for (int i = 0; i < SD_ID_LEN; i++) {
        Serial.print(char(eeFile.read()));
        if (i == 4 || i == 9) {
          Serial.print("-");  
        }
      }
      closeConfig();  
    }
  }
  Serial.print(suffix);
}

void sdConfigCheckId() {
  if (!sdCorrectId()) {
    generateId();
    sdSetId();
  } else {
      sdSetBuffId();
    }
}

// SD config Info

void printSdConfigInfo() {
  sdPrintName(F(" Name: "), "\n");
  //sdPrintId(F(" ID: "), "\n");
  Serial.print(F(" Address: ")); Serial.print(sdGetAddress()); Serial.println();
}
